@@ -1,21 +1,20 @@
 # -*- coding: utf-8 -*-
 #Standard python imports
-from __future__ import division
+from __future__     import division
+try:                import configparser
+except ImportError: import ConfigParser as configparser
 import h5py, matplotlib.pyplot as plt, numpy as np, os, sys
 
 #LVC imports
-from lalinference import DetFrameToEquatorial, EquatorialToDetFrame
+from lalinference                import DetFrameToEquatorial, EquatorialToDetFrame
 from lalinference.imrtgr.nrutils import bbh_final_mass_projected_spins, bbh_final_spin_projected_spins, bbh_Kerr_trunc_opts
-
 import lal, lalsimulation as lalsim
 
 #Package internal imports
 from pyRing.likelihood import project
-from pyRing.utils      import check_NR_dir, qnm_interpolate, qnm_interpolate_KN, resize_time_series, review_warning
+from pyRing.utils      import check_NR_dir, qnm_interpolate, qnm_interpolate_KN, resize_time_series, review_warning, set_prefix
 from pyRing            import waveform as wf
 
-
-
 def inject_ringdown_signal(lenstrain, tstart, length, ifo, triggertime, **kwargs):
 
     """
@@ -41,7 +40,7 @@ def inject_ringdown_signal(lenstrain, tstart, length, ifo, triggertime, **kwargs
         wf_model = khs_injection(**kwargs)
     hs, hvx, hvy, hp, hc = wf_model.waveform(time_wf)[0], wf_model.waveform(time_wf)[1], wf_model.waveform(time_wf)[2], wf_model.waveform(time_wf)[3], wf_model.waveform(time_wf)[4]
 
-    sys.stdout.write('\nInjecting waveform in {}.\n'.format(ifo))
+    sys.stdout.write('* Injecting the `{}` waveform model in the {} detector.\n\n'.format(kwargs['injection-approximant'], ifo))
 
     h        = np.zeros(len(hs))
     srate    = kwargs['sampling-rate']
@@ -51,7 +50,7 @@ def inject_ringdown_signal(lenstrain, tstart, length, ifo, triggertime, **kwargs
     tM_gps   = lal.LIGOTimeGPS(float(triggertime))
     scaling  = kwargs['injection-scaling']
     if not(scaling==1.0):
-        sys.stdout.write('\nApplying a scaling factor {} to the injection.'.format(scaling))
+        sys.stdout.write('* Applying a scaling factor {} to the injection.\n\n'.format(scaling))
 
     if (kwargs['sky-frame']=='detector'):
         tg, ra, dec = DetFrameToEquatorial(lal.cached_detector_by_prefix[kwargs['ref-det']],
@@ -77,8 +76,6 @@ def inject_ringdown_signal(lenstrain, tstart, length, ifo, triggertime, **kwargs
 
 def damped_sinusoids_injection(**kwargs):
 
-    sys.stdout.write('Injecting Damped Sinusoids model.')
-
     wf_model = wf.Damped_sinusoids(kwargs['injection-parameters']['A']  ,
                                    kwargs['injection-parameters']['f']  ,
                                    kwargs['injection-parameters']['tau'],
@@ -88,8 +85,6 @@ def damped_sinusoids_injection(**kwargs):
 
 def morlet_gabor_wavelets_injection(**kwargs):
 
-    sys.stdout.write('Injecting Morlet-Gabor wavelets model.')
-
     wf_model = wf.Morlet_Gabor_wavelets(kwargs['injection-parameters']['A']  ,
                                         kwargs['injection-parameters']['f']  ,
                                         kwargs['injection-parameters']['tau'],
@@ -99,8 +94,6 @@ def morlet_gabor_wavelets_injection(**kwargs):
 
 def kerr_injection(**kwargs):
 
-    sys.stdout.write('Injecting Kerr model.')
-
     t0               = kwargs['injection-parameters']['t0']
     Mf               = kwargs['injection-parameters']['Mf']
     af               = kwargs['injection-parameters']['af']
@@ -125,29 +118,23 @@ def kerr_injection(**kwargs):
     if(kwargs['qnm-fit'] == 0):
         #FIXME: when including 2 amps for each mode, this line will need to be changed
         for (s,l,m,n) in amps.keys():
-            if(charge_flag):
-                interpolate_freq, interpolate_tau = qnm_interpolate_KN(s,l,m,n)
-            else:
-                interpolate_freq, interpolate_tau = qnm_interpolate(s,l,m,n)
+            if(charge_flag): interpolate_freq, interpolate_tau = qnm_interpolate_KN(s,l,m,n)
+            else:            interpolate_freq, interpolate_tau = qnm_interpolate(s,l,m,n)
             qnm_interpolants[(s,l,m,n)] = {'freq': interpolate_freq, 'tau': interpolate_tau}
 
     try:
-        for (s,l,m,n) in domegas.keys():
-            TGR_parameters['domega_{}{}{}'.format(l,m,n)] = domegas[(s,l,m,n)]
-    except:
-        pass
+        for (s,l,m,n) in domegas.keys(): TGR_parameters['domega_{}{}{}'.format(l,m,n)] = domegas[(s,l,m,n)]
+    except: pass
     try:
-        for (s,l,m,n) in dtaus.keys():
-            TGR_parameters['dtau_{}{}{}'.format(l,m,n)] = dtaus[(s,l,m,n)]
-    except:
-        pass
+        for (s,l,m,n) in dtaus.keys():   TGR_parameters['dtau_{}{}{}'.format(l,m,n)] = dtaus[(s,l,m,n)]
+    except: pass
 
     if(area_flag):
         TGR_parameters['alpha'] = kwargs['injection-parameters']['alpha']
-        sys.stdout.write('Injecting a modified Kerr waveform according to the area quantization prescription. alpha: {}'.format(TGR_parameters['alpha']))
+        sys.stdout.write('* Injecting a modified Kerr waveform according to the area quantization prescription. alpha: {}'.format(TGR_parameters['alpha']))
     elif(charge_flag):
         TGR_parameters['Q'] = Q
-        sys.stdout.write('Injecting a KN waveform. Q: {}'.format(TGR_parameters['Q']))
+        sys.stdout.write('* Injecting a KN waveform. Q: {}'.format(TGR_parameters['Q']))
     if not(af**2 + Q**2 < 1):
         raise ValueError("The selected values of charge and spin break the extremality limit (spin = {spin}, charge = {charge} : af^2 + Q^2 = {tot}).".format(spin=af, charge=Q, tot = af**2 + Q**2))
 
@@ -171,8 +158,6 @@ def kerr_injection(**kwargs):
 
 def mmrdns_injection(**kwargs):
 
-    sys.stdout.write('Injecting MMRDNS model.')
-
     t0      = kwargs['injection-parameters']['t0']
     Mf      = kwargs['injection-parameters']['Mf']
     af      = kwargs['injection-parameters']['af']
@@ -206,8 +191,6 @@ def mmrdns_injection(**kwargs):
 
 def mmrdnp_injection(**kwargs):
 
-    sys.stdout.write('Injecting MMRDNP model.')
-
     t0      = kwargs['injection-parameters']['t0']
     m1      = kwargs['injection-parameters']['m1']
     m2      = kwargs['injection-parameters']['m2']
@@ -249,8 +232,6 @@ def mmrdnp_injection(**kwargs):
     return wf_model
 
 def TEOBPM_injection(**kwargs):
-
-    sys.stdout.write('Injecting TEOBResumSPM model.')
     
     multipoles = [(2,2), (3,3), (4,4), (5,5)]
     
@@ -285,8 +266,6 @@ def TEOBPM_injection(**kwargs):
 
 def khs_injection(**kwargs):
 
-    sys.stdout.write('Injecting KHS_2012 model.')
-
     t0      = kwargs['injection-parameters']['t0']
     Mf      = kwargs['injection-parameters']['Mf']
     af      = kwargs['injection-parameters']['af']
@@ -356,7 +335,7 @@ def inject_IMR_signal(lenstrain, tstart, length, ifo, triggertime, **kwargs):
 
     if not (scaling == 1.0):
         dist = dist/scaling
-        sys.stdout.write('\nApplying a scaling factor {} to the injection.'.format(scaling))
+        sys.stdout.write('\n* Applying a scaling factor {} to the injection.'.format(scaling))
 
     if (kwargs['sky-frame']=='detector'):
         tg, ra, dec = DetFrameToEquatorial(lal.cached_detector_by_prefix[kwargs['ref-det']],
@@ -373,14 +352,14 @@ def inject_IMR_signal(lenstrain, tstart, length, ifo, triggertime, **kwargs):
 
     if (kwargs['injection-parameters']['inject-modes'] is not None):
         ModeArray = lalsim.SimInspiralCreateModeArray()
-        sys.stdout.write('\nInjecting a subset of modes: ')
+        sys.stdout.write('\n* Injecting a subset of modes: ')
         for mode in kwargs['injection-parameters']['inject-modes']:
             sys.stdout.write('l={}, m={}; \n'.format(mode[0], mode[1]))
             lalsim.SimInspiralModeArrayActivateMode(ModeArray, mode[0], mode[1])
         lalsim.SimInspiralWaveformParamsInsertModeArray(params, ModeArray)
     elif (kwargs['injection-parameters']['inject-l-modes'] is not None):
         ModeArray = lalsim.SimInspiralCreateModeArray()
-        sys.stdout.write('\nInjecting a subset of l modes (all |m|<l modes are being injected): ')
+        sys.stdout.write('\n* Injecting a subset of l modes (all |m|<l modes are being injected): ')
         for mode in kwargs['injection-parameters']['inject-l-modes']:
             sys.stdout.write('l={}; '.format(mode))
             lalsim.SimInspiralModeArrayActivateAllModesAtL(ModeArray, mode)
