@@ -1,21 +1,45 @@
 #Standard python imports
 from __future__ import division
-from optparse import OptionParser
+from optparse          import OptionParser
 from scipy.interpolate import interp1d, LinearNDInterpolator
-from scipy.linalg import cholesky
-from scipy.signal import butter, filtfilt, tukey
-import numpy as np, pkg_resources, os, traceback, warnings
+from scipy.signal      import butter, filtfilt, tukey
+import numpy as np, pkg_resources, os, scipy.linalg as sl, traceback, warnings
 
 #LVC imports
 import lal
 
 #Package internal imports
-try:
-    import surfinBH
-except:
-    warnings.warn("surfinBH is not automatically installed due to possible conflicts. If you wish to use its functionalities, it needs to be installed separately.")
+try:    import surfinBH
+except: warnings.warn("* The `surfinBH` package is not automatically installed due to possible conflicts. If you wish to use its functionalities, it needs to be installed separately.")
 
 
+def print_section(name):
+
+    pad = "#" * len(name)
+
+    print('\n\n\n##{}##'.format(pad))
+    print('# \u001b[\u001b[38;5;39m{}\u001b[0m #'.format(name))
+    print('##{}##\n'.format(pad))
+
+    return
+
+def print_subsection(name):
+
+    pad = "-" * len(name)
+    
+    print('\n--{}--'.format(pad))
+    print('- \u001b[\u001b[38;5;39m{}\u001b[0m -'.format(name))
+    print('--{}--\n'.format(pad))
+
+    return
+
+def print_out_of_bounds_warning(name):
+    
+    print('\n\n######################### WARNING ############################')
+    print('# The {} injected values are outside the prior bounds. #'.format(name))
+    print('##############################################################\n\n')
+
+    return
 
 def set_prefix(warning_message=True):
     
@@ -32,14 +56,12 @@ def set_prefix(warning_message=True):
             warnings.warn("The requested functionality requires data not included in the package. Please set a $PYRING_PREFIX variable which contains the path to such data. This can be done by setting 'export PYRING_PREFIX= yourpath' in your ~/.bashrc file. Typically, PYRING_PREFIX contains the path to the clone of the repository containing the source code.")
     return prefix
 
-
 def import_datafile_path(filename):
 
     package_path = pkg_resources.resource_filename(__name__, filename)
 
     return package_path
 
-
 def check_NR_dir():
     
     PYRING_PREFIX = set_prefix()
@@ -47,15 +69,14 @@ def check_NR_dir():
         raise Exception("pyRing supports NR injections using the LVC-NR injection infrastructure. If you wish to inject NR simulations, please clone the LVC-NR injection infrastructure repository, located here: https://git.ligo.org/waveforms/lvcnr-lfs \nFor tutorials and info on how to use the LVC NR injection infrastructure see:\n - https://git.ligo.org/sebastian-khan/waveform-f2f-berlin/blob/master/notebooks/2017WaveformsF2FTutorial_NRDemo.ipynb \n - https://www.lsc-group.phys.uwm.edu/ligovirgo/cbcnote/Waveforms/NR/InjectionInfrastructure \n - https://arxiv.org/pdf/1703.01076.pdf")
     return
 
-
 def m1_from_m_q(m, q): return m*q/(1+q)
 def m2_from_m_q(m, q): return m/(1+q)
 
 def review_warning():
-    warnings.warn("\nYou are using a code block which is not reviewed. Non-reviewed code cannot be used for producing LVC results.")
+    print("* Warning: You are using a code block which is not reviewed. Non-reviewed code cannot be used for producing LVC results.")
 
 def qnm_interpolate(s,l,m,n):
-    print('\nPerforming interpolation of ringdown complex frequencies of {}{}{}{} mode.'.format(s,l,m,n))
+    print('* Interpolating ringdown complex frequencies of {}{}{}{} mode.'.format(s,l,m,n))
     assert not(np.abs(m) > l), "QNM interpolation: m cannot be greater than l in modulus."
     assert (s==0 or s==1 or s==2), "QNM interpolation: supported s values are [0,1,2] ({} was passed)."
     try:
@@ -66,7 +87,7 @@ def qnm_interpolate(s,l,m,n):
         else:
             af, w_r, w_i, _, _ = np.loadtxt(os.path.join(PYRING_PREFIX,'data/NR_data/Kerr_BH/s{}l{}'.format(s, l),'n{}l{}m{}.dat'.format(n+1, l, m)), unpack=True)
     except:
-        raise Exception("If you wish to use perturbation theory NR data not stored on the repository, please download the corresponding files from https://pages.jh.edu/~eberti2/ringdown\nQNM interpolation failed with error: {}.".format(traceback.print_exc()))
+        raise Exception("If you wish to use perturbation theory NR data not stored on the repository, please download the corresponding files from `https://pages.jh.edu/~eberti2/ringdown` and place them within directories with the following structure: `pyring_installation_directory/pyring/pyRing/data/NR_data/Kerr_BH/sXlY`, where `X` is the value of the spin perturbation considered and `Y` the value of the `l` QNM index. This feature requires the installation of the source code and is not currently supported by pip.\nQNM interpolation failed with error: {}.".format(traceback.print_exc()))
     return interp1d(af, w_r, kind='cubic'), interp1d(af, w_i, kind='cubic')
 
 def qnm_interpolate_KN(s,l,m,n):
@@ -88,22 +109,10 @@ def qnm_interpolate_KN(s,l,m,n):
 
     return interp_r, interp_i
 
-def whiten(strain, interp_psd, dt, f_min, f_max, mf, alpha_window=0.1):
+def bandpass_around_ringdown(strain, dt, f_min, mf, alpha_window=0.1):
 
-    #########################################################################
-    # Function to whiten the data. Transform to freq domain, divide by asd, #
-    # then transform back, taking care to get normalization right.          #
-    #########################################################################
-
-    Nt       = len(strain)
-    freqs    = np.fft.rfftfreq(Nt, dt)
     srate_dt = 1./dt
-    
-    # This cleaning is required because when we inject a given PSD, the extrapolation sets it to 0 in the region outside the interpolation range. Such a 0 would cause the whitening to crash.
-    psd_cleaned = interp_psd(freqs)
-    for i in range(0, len(psd_cleaned)):
-        if((freqs[i]<f_min) or (freqs[i]>f_max)):
-            psd_cleaned[i] = np.inf
+    Nt       = len(strain)
 
     if not(mf==0.0):
         # Typical ringdown frequency (220 mode) for a BH with af=0.7 (Berti+ fit), only for plotting purposes.
@@ -114,27 +123,78 @@ def whiten(strain, interp_psd, dt, f_min, f_max, mf, alpha_window=0.1):
         bb, ab = butter(4, [f_min/(0.5*srate_dt), (central_f_ringdown*2.)/(0.5*srate_dt)], btype='band')
         strain = filtfilt(bb, ab, strain)
 
+    return strain
+
+def whiten_TD(x, cholesky_L, method='solve-triangular'):
+
+    # If x is a multivariate gaussian variable with covariance C (p(x) ~ x^T * C^{-1} * x), one can use the Cholesky decomposition to obtain C in terms of a lower triangular matrix L: `C = L * L^T`. This implies that `z~N(0,1)` with `z = L^{-1} * x` (below called `x_whitened`), i.e. we need to solve the linear system `L * z = x` for the unknown `z`.
+
+    if(  method=='solve'):            x_whitened = sl.solve(           cholesky_L, x, lower=True, check_finite=False)
+    elif(method=='solve-triangular'): x_whitened = sl.solve_triangular(cholesky_L, x, lower=True, check_finite=False)
+    elif(method=='solve-numpy'):      x_whitened = np.linalg.solve(    cholesky_L, x)
+
+    else:                             raise ValueError('Unknown whitening method requested')
+    
+    return x_whitened
+
+def whiten_FD(strain, interp_psd, dt, f_min, f_max):
+
+    #########################################################################
+    # Function to whiten the data. Transform to freq domain, divide by asd, #
+    # then transform back, taking care to get normalization right.          #
+    #########################################################################
+
+    # Initialise auxiliary quantities
+    Nt       = len(strain)
+    freqs    = np.fft.rfftfreq(Nt, dt)
+    srate_dt = 1./dt
+
+    # Clean PSD. Required because when we inject a given PSD, the extrapolation sets it to 0 in the region outside the interpolation range. Such a 0 would cause the whitening to crash.
+    psd_cleaned = interp_psd(freqs)
+    for i in range(0, len(psd_cleaned)):
+        if((freqs[i]<f_min) or (freqs[i]>f_max)): psd_cleaned[i] = np.inf
+
     hf       = np.fft.rfft(strain)
     white_hf = hf / (np.sqrt(psd_cleaned/dt/2.))
     white_ht = np.fft.irfft(white_hf, n=Nt)
 
     return white_ht
 
-def compute_SNR_TD(h1, h2, InvCov):
-
-    #FIXME: Normalization factors are missing.
-    SNR = 4.*np.dot(h1,np.dot(InvCov,h2))
-
-    return SNR
+def inner_product_TD(h1, h2, InvCov):
+    
+    return np.dot(h1,np.dot(InvCov,h2))
 
 def inner_product_FD(h1, h2, psd, df):
+    
     return 4.0*df*np.sum(np.conj(h1)*h2/psd).real
 
+def compute_SNR_TD(data, template, weights, method='direct-inversion'):
+
+    # These methods have been found to give all identical results (up to the 11th decimal digit of GW150914 SNR at 10M and seglen=0.1).
+    # The computational time hierarchy is: 'direct-inversion'~0.2ms, 'toeplitz-inversion'~0.7ms, 'cholesky-solve-triangular'~4ms.
+    if(method=='direct-inversion'):
+        # In this case weights is C^{-1}, the inverse covariance matrix
+        hh = inner_product_TD(template, template, weights)
+        dh = inner_product_TD(data,     template, weights)
+    elif(method=='cholesky-solve-triangular'):
+        # In this case weights is L, the Cholesky decomposition of the covariance matrix C
+        whiten_h = whiten_TD(template, weights, method='solve-triangular')
+        whiten_d = whiten_TD(data,     weights, method='solve-triangular')
+        hh       = np.dot(whiten_h, whiten_h)
+        dh       = np.dot(whiten_d, whiten_h)
+    elif(method=='toeplitz-inversion'):
+        # In this case weights is ACF, the autocorrelation function from which the covariance matrix C is computed
+        whiten_whiten_h = sl.solve_toeplitz(weights, template, check_finite=False)
+        hh              = np.dot(template, whiten_whiten_h)
+        dh              = np.dot(data,     whiten_whiten_h)
+    else:
+        raise ValueError('Unknown method requested to compute the TD SNR.')
+
+    return dh/np.sqrt(hh)
+
 def compute_SNR_FD(data, template, psd, df):
 
-    SNR = inner_product_FD(data, template, psd, df)/np.sqrt(inner_product_FD(template, template, psd, df))
-
-    return SNR
+    return inner_product_FD(data, template, psd, df)/np.sqrt(inner_product_FD(template, template, psd, df))
 
 def final_state_surfinBH(Mtot, q, chi1, chi2, f_ref):
 
@@ -205,7 +265,7 @@ def chi_eff(q, a1, a2):                       #eta=0.25 -> chi_1=chi_2=0.5 -> ch
     return chi_1*a1 + chi_2*a2
 
 def cholesky_logdet_C(covariance):
-    R = cholesky(covariance)
+    R = sl.cholesky(covariance)
     return 2.*np.sum([np.log(R[i,i]) for i in range(R.shape[0])])
 
 def resize_time_series(inarr, N, dt, starttime, desiredtc):
